@@ -5,10 +5,11 @@
 # Distributed under the terms of the Modified BSD License.
 from __future__ import print_function
 
-import json
 import os
 import sys
-from tornado.ioloop import IOLoop
+import traceback
+
+from copy import copy
 
 from jupyter_core.application import JupyterApp, base_flags, base_aliases
 
@@ -16,8 +17,8 @@ from traitlets import Bool, Unicode
 
 from .commands import (
     install_extension, uninstall_extension, list_extensions,
-    enable_extension, disable_extension,
-    link_package, unlink_package, build, _get_linked_packages
+    enable_extension, disable_extension, check_extension,
+    link_package, unlink_package, build, get_app_version, HERE
 )
 
 
@@ -26,15 +27,24 @@ flags['no-build'] = (
     {'BaseExtensionApp': {'should_build': False}},
     "Defer building the app after the action."
 )
+flags['dev-build'] = (
+    {'BaseExtensionApp': {'dev_build': True}},
+    "Build in Development mode"
+)
+flags['clean'] = (
+    {'BaseExtensionApp': {'should_clean': True}},
+    "Cleanup intermediate files after the action."
+)
 
+check_flags = copy(flags)
+check_flags['installed'] = (
+    {'CheckLabExtensionsApp': {'should_check_installed_only': True}},
+    "Check only if the extension is installed."
+)
 aliases = dict(base_aliases)
 aliases['app-dir'] = 'BaseExtensionApp.app_dir'
 
-
-here = os.path.dirname(__file__)
-with open(os.path.join(here, 'package.app.json')) as fid:
-    data = json.load(fid)
-VERSION = data['jupyterlab']['version']
+VERSION = get_app_version()
 
 
 class BaseExtensionApp(JupyterApp):
@@ -45,9 +55,36 @@ class BaseExtensionApp(JupyterApp):
     app_dir = Unicode('', config=True,
         help="The app directory to target")
 
-    should_build = Bool(False, config=True,
+    should_build = Bool(True, config=True,
         help="Whether to build the app after the action")
 
+    dev_build = Bool(False, config=True,
+        help="Whether to build in dev mode (defaults to production mode)")
+
+    should_clean = Bool(False, config=True,
+        help="Whether temporary files should be cleaned up after building jupyterlab")
+
+    def start(self):
+        if self.app_dir and self.app_dir.startswith(HERE):
+            raise ValueError('Cannot run lab extension commands in core app')
+        try:
+            ans = self.run_task()
+            if ans and self.should_build:
+                command = 'build:prod' if not self.dev_build else 'build'
+                build(app_dir=self.app_dir, clean_staging=self.should_clean,
+                      logger=self.log, command=command)
+        except Exception as ex:
+            _, _, exc_traceback = sys.exc_info()
+            msg = traceback.format_exception(ex.__class__, ex, exc_traceback)
+            for line in msg:
+                self.log.debug(line)
+            self.log.error('\nErrored, use --debug for full output:')
+            self.log.error(msg[-1].strip())
+            sys.exit(1)
+
+    def run_task(self):
+        pass
+
     def _log_format_default(self):
         """A default format for messages"""
         return "%(message)s"
@@ -55,84 +92,67 @@ class BaseExtensionApp(JupyterApp):
 
 class InstallLabExtensionApp(BaseExtensionApp):
     description = "Install labextension(s)"
-    should_build = Bool(True, config=True,
-        help="Whether to build the app after the action")
 
-    def start(self):
+    def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
-        [install_extension(arg, self.app_dir, logger=self.log)
-         for arg in self.extra_args]
-        if self.should_build:
-            try:
-                build(self.app_dir, logger=self.log)
-            except Exception as e:
-                for arg in self.extra_args:
-                    uninstall_extension(arg, self.app_dir, logger=self.log)
-                raise e
+        return any([
+            install_extension(arg, self.app_dir, logger=self.log)
+            for arg in self.extra_args
+        ])
 
 
 class LinkLabExtensionApp(BaseExtensionApp):
     description = """
-    Link labextension(s) or packages.
+    Link local npm packages that are not lab extensions.
 
-    Links a package to the JupyterLab build process.  If the package is
-    an extension, it will also be installed as an extension.  A linked
+    Links a package to the JupyterLab build process. A linked
     package is manually re-installed from its source location when
     `jupyter lab build` is run.
     """
     should_build = Bool(True, config=True,
         help="Whether to build the app after the action")
 
-    def start(self):
+    def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
-        [link_package(arg, self.app_dir, logger=self.log)
-         for arg in self.extra_args]
-        if self.should_build:
-            try:
-                build(self.app_dir, logger=self.log)
-            except Exception as e:
-                for arg in self.extra_args:
-                    unlink_package(arg, self.app_dir, logger=self.log)
-                raise e
+        return any([
+            link_package(arg, self.app_dir, logger=self.log)
+            for arg in self.extra_args
+        ])
 
 
 class UnlinkLabExtensionApp(BaseExtensionApp):
-    description = "Unlink labextension(s) or packages by name or path"
-    should_build = Bool(True, config=True,
-        help="Whether to build the app after the action")
+    description = "Unlink packages by name or path"
 
-    def start(self):
+    def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
-        ans = any([unlink_package(arg, self.app_dir, logger=self.log)
-                   for arg in self.extra_args])
-        if ans and self.should_build:
-            build(self.app_dir, logger=self.log)
+        return any([
+            unlink_package(arg, self.app_dir, logger=self.log)
+            for arg in self.extra_args
+        ])
 
 
 class UninstallLabExtensionApp(BaseExtensionApp):
     description = "Uninstall labextension(s) by name"
-    should_build = Bool(True, config=True,
-        help="Whether to build the app after the action")
 
-    def start(self):
+    def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
-        ans = any([uninstall_extension(arg, self.app_dir, logger=self.log)
-                   for arg in self.extra_args])
-        if ans and self.should_build:
-            build(self.app_dir, logger=self.log)
+        return any([
+            uninstall_extension(arg, self.app_dir, logger=self.log)
+            for arg in self.extra_args
+        ])
 
 
 class ListLabExtensionsApp(BaseExtensionApp):
     description = "List the installed labextensions"
 
-    def start(self):
+    def run_task(self):
         list_extensions(self.app_dir, logger=self.log)
 
 
 class EnableLabExtensionsApp(BaseExtensionApp):
     description = "Enable labextension(s) by name"
 
-    def start(self):
+    def run_task(self):
         [enable_extension(arg, self.app_dir, logger=self.log)
          for arg in self.extra_args]
 
@@ -140,11 +160,29 @@ class EnableLabExtensionsApp(BaseExtensionApp):
 class DisableLabExtensionsApp(BaseExtensionApp):
     description = "Disable labextension(s) by name"
 
-    def start(self):
+    def run_task(self):
         [disable_extension(arg, self.app_dir, logger=self.log)
          for arg in self.extra_args]
 
 
+class CheckLabExtensionsApp(BaseExtensionApp):
+    description = "Check labextension(s) by name"
+    flags = check_flags
+
+    should_check_installed_only = Bool(False, config=True,
+        help="Whether it should check only if the extensions is installed")
+
+    def run_task(self):
+        all_enabled = all(
+            check_extension(
+                arg, self.app_dir,
+                self.should_check_installed_only,
+                logger=self.log)
+            for arg in self.extra_args)
+        if not all_enabled:
+            exit(1)
+
+
 _examples = """
 jupyter labextension list                        # list all configured labextensions
 jupyter labextension install <extension name>    # install a labextension
@@ -166,7 +204,8 @@ class LabExtensionApp(JupyterApp):
         link=(LinkLabExtensionApp, "Link labextension(s)"),
         unlink=(UnlinkLabExtensionApp, "Unlink labextension(s)"),
         enable=(EnableLabExtensionsApp, "Enable labextension(s)"),
-        disable=(DisableLabExtensionsApp, "Disable labextensions(s)")
+        disable=(DisableLabExtensionsApp, "Disable labextension(s)"),
+        check=(CheckLabExtensionsApp, "Check labextension(s)"),
     )
 
     def start(self):
@@ -180,3 +219,6 @@ class LabExtensionApp(JupyterApp):
 
 
 main = LabExtensionApp.launch_instance
+
+if __name__ == '__main__':
+    sys.exit(main())
